/*
    Project: TCTEMP
    Version: V1.4a
    File   : image.h

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#ifndef __IMAGE_H
#define __IMAGE_H

/*
 *  Image file format:
 *
 *  <HEADER_SECTOR><ENTRY_SECTOR 1>...<ENTRY_SECTOR N><DATA_SECTOR 1>...
 */

#define MAX_IMAGE_SECTOR_SIZE 4096

#define IMAGE_HEADER_ID    0x3519afd0
#define IT_END_OF_LIST     0x0000     /* end of list                          */
#define IT_COPY            0x0100     /* copy sector from data area           */
#define IT_MULTIPLE_COPY   0x0200     /* copy multiple sectors from data area */
#define IT_REPEAT          0x0300     /* copy contents of previous item       */
#define IT_FILL            0x0400     /* fill sector with LOBYTE(wType)       */
#define IT_FILL_MAX        0x04ff     /* fill sector with 0xff                */

#define IT_DEFAULT_PATTERN 0xff00     /* pattern type                         */

#pragma pack(push, 1)
typedef struct {
    UINT32 dwId;                 /* IMAGE_HEADER_ID  (= 0x3519afd0)           */
    UINT16 wSectorSize;          /* bytes per sector (512, 1024, 2048, 4096)  */
    UINT16 wPatternType;         /* used pattern (IT_FILL or IT_DEFAULT_PATTERN) */
    UINT64 ullEntries;           /* number of entries                         */
    UINT64 ullSectors;           /* number of non-pattern sectors             */
    UINT64 ullTotalSectors;      /* number of total disk sectors              */
    UINT64 ullDataByteOffset;    /* location of first data byte               */
    UINT64 ullFirstImagedSector; /* first imaged disk sector                  */
    UINT64 ullImagedSectors;     /* number of imaged disk sectors             */
} IMAGE_HEADER, *PIMAGE_HEADER;

typedef struct {
    UINT16 wType;
    UINT16 wReserved[3];
    UINT64 ullOffset;
} IMAGE_ENTRY, *PIMAGE_ENTRY;

typedef struct {
    UINT16 wType;                /* IT_MULTIPLE_COPY or IT_REPEAT             */
    UINT16 wReserved[3];
    UINT64 ullCount;
} IMAGE_LOOP_ENTRY, *PIMAGE_LOOP_ENTRY;

#define IMAGE_ENTRIES_PER_SECTOR(uSectorSize) (uSectorSize / sizeof(IMAGE_ENTRY))

#pragma pack(pop)

#endif
