@echo off

rem ***************************************************************************
rem ** Environment settings                                                  **
rem ***************************************************************************

rem Microsoft Visual Studio .NET 2003 (version 7.1) with SP1 or compatible
set DEVENV=%ProgramFiles%\Microsoft Visual Studio .NET 2003\Common7\IDE\devenv.exe

rem Platform SDK for Windows Server 2003 R2 March 2006 Edition or compatible
set SDK=%ProgramFiles%\Microsoft Platform SDK for Windows Server 2003 R2

rem Windows Server 2003 with SP1 DDK (build 3790.1830) or compatible (must not contain spaces!)
SET DDK=C:\SDK\WINDDK\3790.1830

rem Temporary drive letter
set DRIVE=S:

rem ***************************************************************************
rem ** Run DDK build if this batch file is called by itself                  **
rem ***************************************************************************

if "%1" neq "DDKBUILD" goto :noddkbuild
call :setddkenv %2 %3 %4 %5 %6 %7 %8 %9
build
goto :eof
:noddkbuild

rem ***************************************************************************
rem ** Check environment settings                                            **
rem ***************************************************************************

set ARG0=%~0
if exist "%DEVENV%" goto :devenv_exists
cls
echo.Cannot find Microsoft Visual Studio .NET 2003
set INPUT=
set /P INPUT=Do you want to edit the environment settings (y/n)? 
if "%INPUT%" equ "y" goto :edit_settings
:devenv_exists

if exist "%SDK%\setenv.cmd" goto :sdk_exists
cls
echo.Cannot find Platform SDK for Windows Server 2003
set INPUT=
set /P INPUT=Do you want to edit the environment settings (y/n)? 
if "%INPUT%" equ "y" goto :edit_settings
:sdk_exists

if exist "%DDK%\bin\setenv.bat" goto :ddk_exists
cls
echo.Cannot find Windows Server 2003 with SP1 DDK
set INPUT=
set /P INPUT=Do you want to edit the environment settings (y/n)? 
if "%INPUT%" equ "y" goto :edit_settings
:ddk_exists

rem ***************************************************************************
rem ** Remove output directories                                             **
rem ***************************************************************************

rd /s /q bin
rd /s /q "image\Release"
rd /s /q "misc\Release"
rd /s /q "misc\Release x64"

for /d %%i in (tctemp\obj*) do rd /s /q "%%i"
for /d %%i in (tctemp\crypt\obj*) do rd /s /q "%%i"
for /d %%i in (tctemp\na\obj*) do rd /s /q "%%i"
del "tctemp\build*.log"
del "tctemp\build*.wrn"
del "tctemp\build*.err"

rem ***************************************************************************
rem ** Map current directory to temporary drive letter                       **
rem ***************************************************************************

set HOMEDIR=%CD%
subst %DRIVE% /d
subst %DRIVE% .
if errorlevel 1 goto :invalid_drive
cd /d %drive%\

rem ***************************************************************************
rem ** Compile project                                                       **
rem ***************************************************************************

cls
cd tctemp
echo.Compiling TCTEMP (Windows 2000)...
call :ddkbuild "TCTEMP 1/6" fre W2K
echo.Compiling TCTEMP (Windows XP)...
call :ddkbuild "TCTEMP 2/6" fre WXP
echo.Compiling TCTEMP (Windows XP x64)...
call :ddkbuild "TCTEMP 3/6" fre AMD64 WNET

echo.Compiling TCTEMP DEBUG (Windows 2000)...
call :ddkbuild "TCTEMP 4/6" chk W2K
echo.Compiling TCTEMP DEBUG (Windows XP)...
call :ddkbuild "TCTEMP 5/6" chk WXP
echo.Compiling TCTEMP DEBUG (Windows XP x64)...
call :ddkbuild "TCTEMP 6/6" chk AMD64 WNET
cd..

echo.Compiling IMAGE...
call :build x86 release image\image image
echo.Compiling MISC...
call :build x86 release misc\misc misc
echo.Compiling MISC (x64)...
call :build x64 release misc\misc misc

rem ***************************************************************************
rem ** Copy files to redistributable directories                             **
rem ***************************************************************************

set BIN=bin

md "%BIN%"
md "%BIN%\install"
md "%BIN%\install\bin"
md "%BIN%\install\bin-x64"
copy tctemp\objchk_w2k_x86\i386\tctemp.exe %BIN%\install\bin\tctemp-w2k-chk.bin
copy tctemp\objfre_w2k_x86\i386\tctemp.exe %BIN%\install\bin\tctemp-w2k-fre.bin
copy tctemp\objchk_wxp_x86\i386\tctemp.exe %BIN%\install\bin\tctemp-wxp-chk.bin
copy tctemp\objfre_wxp_x86\i386\tctemp.exe %BIN%\install\bin\tctemp-wxp-fre.bin
copy tctemp\objchk_wnet_amd64\amd64\tctemp.exe %BIN%\install\bin-x64\tctemp-wnet-chk.bin
copy tctemp\objfre_wnet_amd64\amd64\tctemp.exe %BIN%\install\bin-x64\tctemp-wnet-fre.bin
copy image\release\image.exe %BIN%\install\bin\image.exe
copy misc\release\misc.exe %BIN%\install\bin\misc.exe
copy "misc\release x64\misc.exe" %BIN%\install\bin-x64\misc.exe
goto :exit

rem ***************************************************************************
rem ** Build x86|x64 release|debug solution project                          **
rem ***************************************************************************

:build
set SOLUTION=%~3
set PROJECT=%~4
if "%1" equ "x86" goto :x86
if "%1" equ "x64" goto :x64
goto :eof
:x86
if "%2" equ "release" goto :x86_release
if "%2" equ "debug" goto :x86_debug
goto :eof
:x86_release
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Release" /project "%PROJECT%.vcproj"
goto :eof
:x86_debug
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Debug" /project "%PROJECT%.vcproj"
goto :eof
:x64
if "%2" equ "release" goto :x64_release
if "%2" equ "debug" goto :x64_debug
goto :eof
:x64_release
call :setsdkenv /XP64 /RETAIL
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Release x64" /project "%PROJECT%.vcproj" /useenv
goto :eof
:x64_debug
call :setsdkenv /XP64 /DEBUG
"%DEVENV%" "%SOLUTION%.sln" /rebuild "Debug x64" /project "%PROJECT%.vcproj" /useenv
goto :eof

rem ***************************************************************************
rem ** ddkbuild fre|chk AMD64|wxp|w2k                                        **
rem ***************************************************************************

:ddkbuild
set PROJECT=%~1
start "Compiling %PROJECT%..." /w cmd /C "%ARG0%" DDKBUILD %2 %3 %4 %5 %6 %7 %8 %9
goto :eof

rem ***************************************************************************
rem ** Set SDK environment                                                   **
rem ***************************************************************************

:setsdkenv
SET _HOMEDIR=%CD%
cd /d "%SDK%"
call setenv %1 %2 %3 %4 %5 %6 %7 %8 %9 >nul
cd /d "%_HOMEDIR%"
set _HOMEDIR=
goto :eof

rem ***************************************************************************
rem ** Set DDK environment                                                   **
rem ***************************************************************************

:setddkenv
SET _HOMEDIR=%CD%
cd /d "%DDK%\bin"
call setenv %DDK% %1 %2 %3 %4 %5 %6 %7 %8 %9 >nul
cd /d "%_HOMEDIR%"
SET _HOMEDIR=
goto :eof

rem ***************************************************************************
rem ** Edit environment settings                                             **
rem ***************************************************************************
:edit_settings
notepad "%ARG0%"
goto :eof

rem ***************************************************************************
rem ** Error messages                                                        **
rem ***************************************************************************

:invalid_drive
cls
echo.ERROR: Drive letter '%DRIVE%' is not free!
pause
goto :exit

rem ***************************************************************************
rem ** Exit                                                                  **
rem ***************************************************************************
:exit
subst %DRIVE% /d
cd /d "%HOMEDIR%"
