/*
    Project: TCTEMP
    Version: V1.4a
    File   : tc_header.c

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "tomcrypt.h"
#include "tc_priv.h"

BOOL __stdcall TC_EncryptHeader(PTC_HEADER pHeader, const void *pPassword,
  size_t cbPasswordSize, unsigned uHash, unsigned uCipher, PTC_BUFFER pBuffer)
{
    BOOL fResult = FALSE;

    if (!TC_HashIsValid(uHash) || !TC_CipherIsValid(uCipher)) {
        return FALSE;
    }

    if (TC_DeriveNonWeakKey(&pBuffer->key, pPassword, cbPasswordSize,
      pHeader->rgbSalt, sizeof(pHeader->rgbSalt), uHash, 0, uCipher,
      &pBuffer->pkcs5_2_state)) {
        if (TCE_Init(&pBuffer->ctx, &pBuffer->key, uCipher)) {

            memset(pHeader->rgbReserved, '\0', sizeof(pHeader->rgbReserved));
            pHeader->dwId = TC_HEADER_ID;
            Reverse32(&pHeader->dwId);
            pHeader->wHeaderVersion = TC_HEADER_VERSION;
            Reverse16(&pHeader->wHeaderVersion);
            pHeader->wReqProgramVersion = TC_REQ_PROGRAM_VERSION;
            Reverse16(&pHeader->wReqProgramVersion);
            pHeader->dwCRC32 = GetCRC32(pHeader->rgbSecondaryKey, 256);
            Reverse32(&pHeader->dwCRC32);

            if (TCE_Encrypt(&pBuffer->ctx, &pHeader->dwId, 512-64)) {
                fResult = TRUE;
            }

            TCE_Close(&pBuffer->ctx);
        }
    }

    return fResult;
}
