#ifndef __NT_API_H
#define __NT_API_H

#include <ntddk.h>
#ifndef DBG
#  define DBG 0
#endif

/*----------------------------------------------------------------------------*/
/* Heap                                                                       */

#define HEAP_GROWABLE    2
#define HEAP_ZERO_MEMORY 8

HANDLE NTAPI RtlCreateHeap(ULONG Flags, PVOID HeapBase, SIZE_T ReserveSize, 
  SIZE_T CommitSize, PVOID Lock, PVOID Parameters);

PVOID NTAPI RtlAllocateHeap(PVOID HeapHandle, ULONG Flags, SIZE_T Size);

/*----------------------------------------------------------------------------*/
/* System Information                                                         */

typedef int SYSTEM_INFORMATION_CLASS;

NTSTATUS NTAPI NtQuerySystemInformation(
  SYSTEM_INFORMATION_CLASS SystemInformationClass, PVOID SystemInformation, 
  ULONG SystemInformationLength, PULONG ReturnLength);

/*----------------------------------------------------------------------------*/
/* Objects                                                                    */

NTSTATUS NTAPI NtClose(HANDLE Handle);

/*----------------------------------------------------------------------------*/
/* Synchronization                                                            */

NTSTATUS NTAPI NtCreateEvent(PHANDLE EventHandle, ACCESS_MASK DesiredAccess,
  POBJECT_ATTRIBUTES ObjectAttributes, EVENT_TYPE EventType, 
  BOOLEAN InitialState);

NTSTATUS NTAPI NtWaitForSingleObject(HANDLE Handle, BOOLEAN Alertable,
  PLARGE_INTEGER Timeout);

/*----------------------------------------------------------------------------*/
/* File                                                                       */

#define FSCTL_CODE(f, m, a)          CTL_CODE(FILE_DEVICE_FILE_SYSTEM, f, m, a)
#define FSCTL_LOCK_VOLUME            FSCTL_CODE( 6, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define FSCTL_UNLOCK_VOLUME          FSCTL_CODE( 7, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define FSCTL_DISMOUNT_VOLUME        FSCTL_CODE( 8, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define FSCTL_ALLOW_EXTENDED_DASD_IO FSCTL_CODE(32, METHOD_NEITHER,  FILE_ANY_ACCESS)

NTSTATUS NTAPI NtCreateFile(PHANDLE FileHandle, ACCESS_MASK DesiredAccess, 
  POBJECT_ATTRIBUTES ObjectAttributes, PIO_STATUS_BLOCK IoStatusBlock,
  PLARGE_INTEGER AllocationSize, ULONG FileAttributes, ULONG ShareAccess, 
  ULONG CreateDisposition, ULONG CreateOptions, PVOID EaBuffer, ULONG EaLength);

NTSTATUS NTAPI NtReadFile(HANDLE FileHandle, HANDLE Event, 
  PIO_APC_ROUTINE ApcRoutine, PVOID ApcContext, PIO_STATUS_BLOCK IoStatusBlock,
  PVOID Buffer, ULONG Length, PLARGE_INTEGER ByteOffset, PULONG Key);

NTSTATUS NTAPI NtWriteFile(HANDLE FileHandle, HANDLE Event, 
  PIO_APC_ROUTINE ApcRoutine, PVOID ApcContext, PIO_STATUS_BLOCK IoStatusBlock,
  PVOID Buffer, ULONG Length, PLARGE_INTEGER ByteOffset, PULONG Key);
 
NTSTATUS NTAPI NtDeviceIoControlFile(HANDLE FileHandle, HANDLE Event,
  PIO_APC_ROUTINE ApcRoutine, PVOID AbcContext, PIO_STATUS_BLOCK IoStatusBlock,
  ULONG IoControlCode, PVOID InputBuffer, ULONG InputBufferLength, 
  PVOID OutputBuffer, ULONG OutputBufferLength);

NTSTATUS NTAPI NtFsControlFile(HANDLE FileHandle, HANDLE Event,
  PIO_APC_ROUTINE ApcRoutine, PVOID AbcContext, PIO_STATUS_BLOCK IoStatusBlock,
  ULONG FsControlCode, PVOID InputBuffer, ULONG InputBufferLength, 
  PVOID OutputBuffer, ULONG OutputBufferLength);

NTSTATUS NTAPI NtQueryInformationFile(HANDLE FileHandle, 
  PIO_STATUS_BLOCK IoStatusBlock, PVOID FileInformation, ULONG Length,
  FILE_INFORMATION_CLASS FileInformationClass);

NTSTATUS NTAPI NtSetInformationFile(HANDLE FileHandle, 
  PIO_STATUS_BLOCK IoStatusBlock, PVOID FileInformation, ULONG Length,
  FILE_INFORMATION_CLASS FileInformationClass);

/*----------------------------------------------------------------------------*/
/* Miscellany                                                                 */

NTSTATUS NTAPI NtDisplayString(PUNICODE_STRING String);

#endif
