/*
    Project: TCTEMP
    Version: V1.4a
    File   : na_rng.c

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "na.h"

#define RNGT_NONE 0
#define RNGT_VIA  1

#define RNGR_UNSUCCESSFUL    -1
#define RNGR_NOT_IMPLEMENTED -2

#define GetNextAlignedPointer(p, a) \
  (PVOID) ( (((size_t)(p)) + ((a)-1)) & ~((a)-1) )

#ifndef _WIN64

#define xstore __asm _emit 0x0f __asm _emit 0xa7 __asm _emit 0xc0

static int get_rng_type(void)
{
    int iRngType = RNGT_NONE;

    /* VIA */
    __asm {
        mov     eax, 0xc0000000
        cpuid
        cmp     eax, 0xc0000001
        jb      no_via_rng
        mov     eax, 0xc0000001
        cpuid
        and     edx, 0x0000000c
        cmp     edx, 0x0000000c
        jne     no_via_rng
        mov     iRngType, RNGT_VIA
    }
    no_via_rng:

    return iRngType;
}

static int get_rng_byte(int iRngType)
{
    unsigned __int8 rgbBuffer[32];
    unsigned __int8 *pbBuffer;
    int             iResult;

    pbBuffer = GetNextAlignedPointer(rgbBuffer, 16);

    if (iRngType == RNGT_VIA) {

        do {

            __asm {
                mov     edi, pbBuffer
                mov     edx, 3
                xstore
                and     eax, 0x0000000f
                jz      unsuccessful
            }

            iResult = *pbBuffer;
            break;

        unsuccessful:
            iResult = RNGR_UNSUCCESSFUL;

        } while (0);

    } else {

        iResult = RNGR_NOT_IMPLEMENTED;

    }

    return iResult;
}

#else

#define get_rng_type()         RNGT_NONE
#define get_rng_byte(iRngType) RNGR_NOT_IMPLEMENTED

#endif

#ifndef get_rng_type

int get_rng_bytes(unsigned __int8 *pbBuffer, size_t cbBufferSize)
{
    int iResult = 0;

    __try {

        unsigned __int8 *pbBufferEnd;
        int             iRngType;

        iRngType = get_rng_type();
        if (iRngType == RNGT_NONE) {
            goto error;
        }

        pbBufferEnd = pbBuffer + cbBufferSize;

        while (pbBuffer != pbBufferEnd) {

            int i;

            i = get_rng_byte(iRngType);
            if (i == RNGR_UNSUCCESSFUL) {
                i = get_rng_byte(iRngType);
                if (i == RNGR_UNSUCCESSFUL) {
                    NaSleep(200);
                    i = get_rng_byte(iRngType);
                }
            }
            if (i < 0) {
                goto error;
            }

            *pbBuffer++ = (unsigned __int8) i;
        }

        iResult = 1;

    error:
        ;

    } __except(EXCEPTION_EXECUTE_HANDLER) {
    }

    return iResult;
}

#else

int get_rng_bytes(unsigned __int8 *pbBuffer, size_t cbBufferSize)
{
    return 0;
}

#endif
