/*
    Project: TCTEMP
    Version: V1.4a
    File   : na_tc.c

    Copyright (c) 2006 Author of TCTEMP. All rights reserved.

    Covered by TrueCrypt License 2.1 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the following public key
    must be retained in full.

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.2.1 (MingW32)

mQGiBEP09h0RBAC1RkmI9S9JfZ0yP+i1j3GgOE+49a7vDCLmIRHEk8KfDR31d2iC
WsjvVWGPSkl0GWfJeWiIU2xOaFjo7zF0Os1nICG1x0hj2p6O3FRjBCPQEFjuXPmV
OexI9DiT/KtCvqjtWTnLwSWBCtH165ond/Aloe2TuNbiCWDoO5VyPNm6DwCg6QHF
bwnTp74YgYsuCVyMI1G5UeUD/AylpfON3ndCqgFgrZm4nJ93qnaSHP/fj/83yqDi
6xh339pqfzoyPMyP85q2QxNQ7PrOKfTHmsrXVj3zzfIz0VQxp7hflxzE6ydJaIzD
YpKZO5qxTRXvMqUxFo275MinDcIMeu2YUOWHnQgFQuZSuQAfsVGxSv4MFxPUQF+G
5/xCA/9wcVsBaGBXkaqVpMbeRx5+UUohEaEMuHPk0Lj6/rGlvAQpsVvuLZ3zO6C2
X7RXtsH8DfwqBAMhNkTqv9RmjUXsryGhrrDgxq7agjJPpbbIfgp/S3OYnwOx1BTb
OwqpTuTcqAJlDtCk3WOuMDa4h6DQeqb9NpIUM/lhkrgjIRqhJ7QQQXV0aG9yIG9m
IFRDVEVNUIhgBBMRAgAgBQJD9PYdAhsDBgsJCAcDAgQVAggDBBYCAwECHgECF4AA
CgkQSl/DkxnubmlEZACbBr6Fsvq5XGgxxHSiZ/DpoMDpxLIAn16QzZeLlLOjb7/I
TzCAdffqb4kW
=61fR
-----END PGP PUBLIC KEY BLOCK-----
*/

#include "na.h"
#include "na_tc.h"

/*
 * The interface to the TrueCrypt device driver is taken from the source
 * code of TrueCrypt 4.2a
 */

#define TC_MAX_PASSWORD_LEN         64
#define TC_MAX_PASSWORD             (TC_MAX_PASSWORD_LEN + 1)
#define TC_MIN_DRIVER_VERSION       0x042a
#define TC_MAX_DRIVER_VERSION       0x042a
#define TC_MAX_PATH                 260
#define TC_IOCTL(idx)               (0x72000 | ((idx) << 2))
#define TC_IOCTL_MOUNT              TC_IOCTL(0)
#define TC_IOCTL_DISMOUNT           TC_IOCTL(3)
#define TC_IOCTL_GET_DRIVER_VERSION TC_IOCTL(6)

typedef struct
{
    INT    nPasswordLen;
    CHAR   aszPassword[TC_MAX_PASSWORD];
} TC_PASSWORD, *PTC_PASSWORD;

#pragma pack(push, 1)

typedef struct
{
    INT         iResult;
    WCHAR       wszVolumeName[TC_MAX_PATH];
    TC_PASSWORD pwVolume;
    BOOL        fCache;
    INT         nDosDriveNo;
    BOOL        fSystemVolume;
    BOOL        fPersistentVolume;
    BOOL        fMountReadOnly;
    BOOL        fMountRemovable;
    BOOL        fExclusiveAccess;
    BOOL        fMountManager;
    BOOL        fUserContext;
    BOOL        fPreserveTimestamp;
    BOOL        fProtectHiddenVolume;
    TC_PASSWORD pwHiddenVolume;
} TC_MOUNT_DATA, *PTC_MOUNT_DATA;

typedef struct
{
    INT    nDosDriveNo;
    BOOL   fIgnoreOpenFiles;
    INT    iResult;
} TC_DISMOUNT_DATA, *PTC_DISMOUNT_DATA;

#pragma pack(pop)

BOOL TCAPI NaTcOpen(PTC_CONTEXT pCtx)
{
    DWORD dwVersion;

    if (!NaOpenDevice(&pCtx->hDevice, L"\\Device\\TrueCrypt")) {
        return FALSE;
    }

    dwVersion = 0;
    if (!NaDeviceIoControl(pCtx->hDevice, TC_IOCTL_GET_DRIVER_VERSION,
      &dwVersion, sizeof(dwVersion), &dwVersion, sizeof(dwVersion), NULL)) {
        NaDbgPrintf(L"Cannot query TrueCrypt driver version!");
        NaCloseFile(pCtx->hDevice);
        return FALSE;
    }
    if (dwVersion < TC_MIN_DRIVER_VERSION ||
      dwVersion > TC_MAX_DRIVER_VERSION) {
        NaDbgPrintf(L"Unsupported TrueCrypt version (%x)!", (unsigned)dwVersion);
        NaCloseFile(pCtx->hDevice);
        return FALSE;
    }

    return TRUE;
}

void TCAPI NaTcClose(PTC_CONTEXT pCtx)
{
    NaCloseFile(pCtx->hDevice);
}

BOOL TCAPI NaTcMountVolume(PTC_CONTEXT pCtx, WCHAR chDriveLetter,
  LPCWSTR pszVolumeName, LPCSTR pPassword, DWORD dwPasswordLength)
{
    PTC_MOUNT_DATA ptmd = (PTC_MOUNT_DATA)pCtx->rgbBuffer;

    NaAssert(pszVolumeName);
    NaAssert(sizeof(TC_MOUNT_DATA) <= TC_BUFFER_SIZE);

    if (wcslen(pszVolumeName) >= sizeof(ptmd->wszVolumeName)/sizeof(WCHAR)) {
        NaDbgPrintf(L"TrueCrypt volume name is too long!");
        return FALSE;
    }
    if (dwPasswordLength) {
        NaAssert(pPassword);
        if (dwPasswordLength > TC_MAX_PASSWORD_LEN) {
            NaDbgPrintf(L"TrueCrypt password is too long!");
            return FALSE;
        }
    }

    RtlZeroMemory(ptmd, sizeof(*ptmd));
    wcscpy(ptmd->wszVolumeName, pszVolumeName);
    if (dwPasswordLength) {
        ptmd->pwVolume.nPasswordLen = dwPasswordLength;
        memcpy(ptmd->pwVolume.aszPassword, pPassword, dwPasswordLength);
    }
    ptmd->nDosDriveNo = towupper(chDriveLetter) - L'A';
    ptmd->fSystemVolume = TRUE;
    ptmd->fExclusiveAccess = TRUE;
    ptmd->fPreserveTimestamp = TRUE;
    if (!NaDeviceIoControl(pCtx->hDevice, TC_IOCTL_MOUNT, ptmd, sizeof(*ptmd),
      ptmd, sizeof(*ptmd), NULL)) {
        NaDbgPrintf(L"Cannot mount TrueCrypt volume!");
        return FALSE;
    } else if (ptmd->iResult != 0) {
        NaDbgPrintf(L"Cannot mount TrueCrypt volume (Error code 0x%x)!",
          ptmd->iResult);
        return FALSE;
    }

    return TRUE;
}

BOOL TCAPI NaTcDismountVolume(PTC_CONTEXT pCtx, WCHAR chDriveLetter)
{
    PTC_DISMOUNT_DATA ptdd = (PTC_DISMOUNT_DATA)pCtx->rgbBuffer;

    NaAssert(sizeof(TC_DISMOUNT_DATA) <= TC_BUFFER_SIZE);

    RtlZeroMemory(ptdd, sizeof(*ptdd));
    ptdd->nDosDriveNo = towupper(chDriveLetter) - L'A';
    ptdd->fIgnoreOpenFiles = TRUE;

    if (!NaDeviceIoControl(pCtx->hDevice, TC_IOCTL_DISMOUNT, ptdd,
      sizeof(*ptdd), ptdd, sizeof(*ptdd), NULL)) {
        NaDbgPrintf(L"Cannot dismount TrueCrypt volume!");
        return FALSE;
    } else if (ptdd->iResult != 0) {
        NaDbgPrintf(L"Cannot dismount TrueCrypt volume (Error code 0x%x)!",
          ptdd->iResult);
        return FALSE;
    }

    return TRUE;
}
